@@ -5,7 +5,7 @@
 
 
 #define RAV1E_MAJOR 0
-#define RAV1E_MINOR 4
+#define RAV1E_MINOR 6
 #define RAV1E_PATCH 1
 
 
@@ -262,7 +262,7 @@ typedef enum {
 /**
  * Allowed pixel value range
  *
- * C.f. VideoFullRangeFlag variable specified in ISO/IEC 23091-4/ITU-T H.273
+ * C.f. `VideoFullRangeFlag` variable specified in ISO/IEC 23091-4/ITU-T H.273
  */
 typedef enum {
     /**
@@ -300,7 +300,7 @@ typedef enum {
     /**
      * There is no pass data available for now
      *
-     * This is emitted if rav1e_rc_receive_pass_data is called more
+     * This is emitted if `rav1e_rc_receive_pass_data` is called more
      * often than it should.
      */
     RA_RC_DATA_KIND_EMPTY,
@@ -385,37 +385,36 @@ typedef enum {
 /**
  * Encoder configuration
  *
- * Instantiate it using rav1e_config_default() and fine-tune it using
- * rav1e_config_parse().
+ * Instantiate it using `rav1e_config_default()` and fine-tune it using
+ * `rav1e_config_parse()`.
  *
- * Use rav1e_config_unref() to free its memory.
+ * Use `rav1e_config_unref()` to free its memory.
  */
 typedef struct RaConfig RaConfig;
 
 /**
  * Encoder context
  *
- * Contains the encoding state, it is created by rav1e_context_new() using an
+ * Contains the encoding state, it is created by `rav1e_context_new()` using an
  * Encoder configuration.
  *
- * Use rav1e_context_unref() to free its memory.
+ * Use `rav1e_context_unref()` to free its memory.
  */
 typedef struct RaContext RaContext;
 
 /**
  * Raw video Frame
  *
- * It can be allocated through rav1e_frame_new(), populated using rav1e_frame_fill_plane()
- * and freed using rav1e_frame_unref().
+ * It can be allocated through `rav1e_frame_new()`,
+ * populated using `rav1e_frame_fill_plane()`,
+ * and freed using `rav1e_frame_unref()`.
  */
 typedef struct RaFrame RaFrame;
 
 /**
  * Simple Data
  *
- *
- *
- * Use rav1e_data_unref() to free its memory.
+ * Use `rav1e_data_unref()` to free its memory.
  */
 typedef struct {
     /**
@@ -462,9 +461,9 @@ typedef void (*RaFrameOpaqueCb)(void*);
 /**
  * Encoded Packet
  *
- * The encoded packets are retrieved using rav1e_receive_packet().
+ * The encoded packets are retrieved using `rav1e_receive_packet()`.
  *
- * Use rav1e_packet_unref() to free its memory.
+ * Use `rav1e_packet_unref()` to free its memory.
  */
 typedef struct {
     /**
@@ -487,6 +486,16 @@ typedef struct {
      * User provided opaque data
      */
     void *opaque;
+    /**
+     * The reconstruction of the shown frame.
+     * This is freed automatically by `rav1e_packet_unref()`.
+     */
+    RaFrame *rec;
+    /**
+     * The Reference Frame
+     * This is freed automatically by `rav1e_packet_unref()`.
+     */
+    RaFrame *source;
 } RaPacket;
 
 #ifdef __cplusplus
@@ -496,7 +505,7 @@ extern "C" {
 /**
  * Version information as presented in `[package]` `version`.
  *
- * e.g. `0.1.0``
+ * e.g. `0.1.0`
  *
  * Can be parsed by [semver](https://crates.io/crates/semver).
  * This returns the version of the loaded library, regardless
@@ -516,27 +525,27 @@ const char *rav1e_version_short(void);
 const char *rav1e_version_full(void);
 
 /**
- * Free a RaData buffer
+ * Free a `RaData` buffer
  */
 void rav1e_data_unref(RaData *data);
 
 /**
- * Create a RaConfig filled with default parameters.
+ * Create a `RaConfig` filled with default parameters.
  */
 RaConfig *rav1e_config_default(void);
 
 /**
  * Setup a second pass rate control using the provided summary
  *
- * Passing NULL data resets the rate control settings.
+ * Passing `NULL` data resets the rate control settings.
  *
  * If additional data is required, pointer and len stay unchanged, otherwise
  * they are updated.
  *
  * Return:
- * 0 on success
- * > 0 if the buffer has to be larger
- * < 0 on failure
+ * `0` on success
+ * `> 0` if the buffer has to be larger
+ * `< 0` on failure
  */
 int rav1e_config_set_rc_summary(RaConfig *cfg, const uint8_t **data, size_t *len);
 
@@ -565,9 +574,9 @@ void rav1e_config_set_time_base(RaConfig *cfg, RaRational time_base);
 /**
  * Set pixel format of the stream.
  *
- * Supported values for subsampling and chromapos are defined by the
- * enum types RaChromaSampling and RaChromaSamplePosition respectively.
- * Valid values for fullrange are 0 and 1.
+ * Supported values for `subsampling` and `chroma_pos` are defined by the
+ * enum types `RaChromaSampling` and `RaChromaSamplePosition` respectively.
+ * Valid values for `pixel_range` are 0 and 1.
  *
  * Returns a negative value on error or 0.
  */
@@ -581,7 +590,7 @@ int rav1e_config_set_pixel_format(RaConfig *cfg,
  * Set color properties of the stream.
  *
  * Supported values are defined by the enum types
- * RaMatrixCoefficients, RaColorPrimaries, and RaTransferCharacteristics
+ * `RaMatrixCoefficients`, `RaColorPrimaries`, and `RaTransferCharacteristics`
  * respectively.
  *
  * Return a negative value on error or 0.
@@ -603,10 +612,10 @@ int rav1e_config_set_content_light(RaConfig *cfg,
 /**
  * Set the mastering display information for HDR10 streams.
  *
- * primaries and white_point arguments are RaChromaticityPoint, containing 0.16 fixed point
- * values.
- * max_luminance is a 24.8 fixed point value.
- * min_luminance is a 18.14 fixed point value.
+ * `primaries` and `white_point` arguments are `RaChromaticityPoint`,
+ * containing 0.16 fixed point values.
+ * `max_luminance` is a 24.8 fixed point value.
+ * `min_luminance` is a 18.14 fixed point value.
  *
  * Returns a negative value on error or 0.
  */
@@ -617,7 +626,7 @@ int rav1e_config_set_mastering_display(RaConfig *cfg,
                                        uint32_t min_luminance);
 
 /**
- * Free the RaConfig.
+ * Free the `RaConfig`.
  */
 void rav1e_config_unref(RaConfig *cfg);
 
@@ -625,25 +634,25 @@ void rav1e_config_unref(RaConfig *cfg);
  * Set a configuration parameter using its key and value as string.
  *
  * Available keys and values
- * - "width": width of the frame, default 640
- * - "height": height of the frame, default 480
- * - "speed": 0-10, default 6
- * - "threads": maximum number of threads to be used
- * - "tune": "psnr"-"psychovisual", default "psychovisual"
- * - "quantizer": 0-255, default 100
- * - "tiles": total number of tiles desired (0 denotes auto), default 0
- * - "tile_rows": number of tiles horizontally (must be a power of two, overridden by tiles if present), default 0
- * - "tile_cols": number of tiles vertically (must be a power of two, overridden by tiles if present), default 0
- * - "min_quantizer": minimum allowed base quantizer to use in bitrate mode, default 0
- * - "bitrate": target bitrate for the bitrate mode (required for two pass mode), default 0
- * - "key_frame_interval": maximum interval between two keyframes, default 240
- * - "min_key_frame_interval": minimum interval between two keyframes, default 12
- * - "switch_frame_interval": interval between switch frames, default 0
- * - "reservoir_frame_delay": number of temporal units over which to distribute the reservoir usage, default None
- * - "rdo_lookahead_frames": number of frames to read ahead for the RDO lookahead computation, default 40
- * - "low_latency": flag to enable low latency mode, default false
- * - "enable_timing_info": flag to enable signaling timing info in the bitstream, default false
- * - "still_picture": flag for still picture mode, default false
+ * - `"width"`: width of the frame, default `640`
+ * - `"height"`: height of the frame, default `480`
+ * - `"speed"`: 0-10, default `6`
+ * - `"threads"`: maximum number of threads to be used, default auto
+ * - `"tune"`: `"psnr"` or `"psychovisual"`, default `"psychovisual"`
+ * - `"quantizer"`: 0-255, default `100`
+ * - `"tiles"`: total number of tiles desired (0 denotes auto), default `0`
+ * - `"tile_rows"`: number of tiles horizontally (must be a power of two, overridden by tiles if present), default `0`
+ * - `"tile_cols"`: number of tiles vertically (must be a power of two, overridden by tiles if present), default `0`
+ * - `"min_quantizer"`: minimum allowed base quantizer to use in bitrate mode, default `0`
+ * - `"bitrate"`: target bitrate for the bitrate mode (required for two pass mode), default `0`
+ * - `"key_frame_interval"`: maximum interval between two keyframes, default `240`
+ * - `"min_key_frame_interval"`: minimum interval between two keyframes, default `12`
+ * - `"switch_frame_interval"`: interval between switch frames, default `0`
+ * - `"reservoir_frame_delay"`: number of temporal units over which to distribute the reservoir usage, default `None`
+ * - `"rdo_lookahead_frames"`: number of frames to read ahead for the RDO lookahead computation, default `40`
+ * - `"low_latency"`: flag to enable low latency mode, default `false`
+ * - `"enable_timing_info"`: flag to enable signaling timing info in the bitstream, default `false`
+ * - `"still_picture"`: flag for still picture mode, default `false`
  *
  * Return a negative value on error or 0.
  */
@@ -654,7 +663,7 @@ int rav1e_config_parse(RaConfig *cfg,
 /**
  * Set a configuration parameter using its key and value as integer.
  *
- * Available keys and values are the same as rav1e_config_parse()
+ * Available keys and values are the same as `rav1e_config_parse()`
  *
  * Return a negative value on error or 0.
  */
@@ -664,35 +673,35 @@ int rav1e_config_parse_int(RaConfig *cfg, const char *key, int value);
  * Generate a new encoding context from a populated encoder configuration
  *
  * Multiple contexts can be generated through it.
- * Returns Null if context creation failed, e.g. by passing
- * an invalid Config.
+ * Returns `Null` if context creation failed, e.g. by passing
+ * an invalid `Config`.
  */
 RaContext *rav1e_context_new(const RaConfig *cfg);
 
 /**
- * Free the RaContext.
+ * Free the `RaContext`.
  */
 void rav1e_context_unref(RaContext *ctx);
 
 /**
  * Produce a new frame from the encoding context
  *
- * It must be populated using rav1e_frame_fill_plane().
+ * It must be populated using `rav1e_frame_fill_plane()`.
  *
- * The frame is reference counted and must be released passing it to rav1e_frame_unref(),
- * see rav1e_send_frame().
+ * The frame is reference counted and must be released passing it to `rav1e_frame_unref()`,
+ * see `rav1e_send_frame()`.
  */
 RaFrame *rav1e_frame_new(const RaContext *ctx);
 
 /**
- * Free the RaFrame.
+ * Free the `RaFrame`.
  */
 void rav1e_frame_unref(RaFrame *frame);
 
 /**
  * Overrides the encoders frame type decision for a frame
  *
- * Must be called before rav1e_send_frame() if used.
+ * Must be called before `rav1e_send_frame()` if used.
  */
 int rav1e_frame_set_type(RaFrame *frame, RaFrameTypeOverride frame_type);
 
@@ -701,27 +710,42 @@ int rav1e_frame_set_type(RaFrame *frame, RaFrameTypeOverride frame_type);
  *
  * It takes the ownership of its memory:
  * - it will relinquish the ownership to the context if
- *   rav1e_send_frame is called.
- * - it will call the destructor if rav1e_frame_unref is called
+ *   `rav1e_send_frame` is called.
+ * - it will call the destructor if `rav1e_frame_unref` is called
  *   otherwise.
  */
 void rav1e_frame_set_opaque(RaFrame *frame, void *opaque, RaFrameOpaqueCb cb);
 
+/**
+ * Add generic T35 metadata to a frame
+ *
+ * The buffer will be copied into the frame and can be freed
+ * immediately after this call.
+ *
+ * Can be called multiple times to add multiple T35 metadata
+ * blocks.
+ */
+void rav1e_frame_add_t35_metadata(RaFrame *frame,
+                                  uint8_t country_code,
+                                  uint8_t country_code_extension_byte,
+                                  const uint8_t *data,
+                                  size_t data_len);
+
 /**
  * Retrieve the first-pass data of a two-pass encode for the frame that was
- * just encoded. This should be called BEFORE every call to rav1e_receive_packet()
+ * just encoded. This should be called BEFORE every call to `rav1e_receive_packet()`
  * (including the very first one), even if no packet was produced by the
- * last call to rav1e_receive_packet, if any (i.e., RA_ENCODER_STATUS_ENCODED
+ * last call to `rav1e_receive_packet`, if any (i.e., `RA_ENCODER_STATUS_ENCODED`
  * was returned). It needs to be called once more after
- * RA_ENCODER_STATUS_LIMIT_REACHED is returned, to retrieve the header that
+ * `RA_ENCODER_STATUS_LIMIT_REACHED` is returned, to retrieve the header that
  * should be written to the front of the stats file (overwriting the
  * placeholder header that was emitted at the start of encoding).
  *
- * It is still safe to call this function when rav1e_receive_packet() returns any
- * other error. It will return NULL instead of returning a duplicate copy
+ * It is still safe to call this function when `rav1e_receive_packet()` returns any
+ * other error. It will return `NULL` instead of returning a duplicate copy
  * of the previous frame's data.
  *
- * Must be freed with rav1e_data_unref().
+ * Must be freed with `rav1e_data_unref()`.
  */
 RaData *rav1e_twopass_out(RaContext *ctx);
 
@@ -736,12 +760,12 @@ size_t rav1e_rc_summary_size(const RaContext *ctx);
 /**
  * Return the first pass data
  *
- * Call it after rav1e_receive_packet() returns a normal condition status:
- * EncoderStatus::Encoded,
- * EncoderStatus::Success,
- * EncoderStatus::LimitReached.
+ * Call it after `rav1e_receive_packet()` returns a normal condition status:
+ * - `EncoderStatus::Encoded`,
+ * - `EncoderStatus::Success`,
+ * - `EncoderStatus::LimitReached`.
  *
- * use rav1e_data_unref() to free the data.
+ * use `rav1e_data_unref()` to free the data.
  *
  * It will return a `RcDataKind::Summary` once the encoder is flushed.
  */
@@ -773,7 +797,7 @@ int32_t rav1e_rc_second_pass_data_required(const RaContext *ctx);
  * Feed the first pass Rate Control data to the encoder,
  * Frame-specific Packets only.
  *
- * Call it before receive_packet()
+ * Call it before `receive_packet()`
  *
  * If additional data is required, pointer and len stay unchanged, otherwise
  * they are updated.
@@ -791,8 +815,8 @@ int rav1e_rc_send_pass_data(RaContext *ctx, const uint8_t **data, size_t *len);
  * bound (more might be required), but if 0 is returned, then encoding can
  * proceed. This is just a hint to the application, and does not need to
  * be called for encoding the second pass to work, so long as the
- * application continues to provide more data to rav1e_twopass_in() in a loop
- * until rav1e_twopass_in() returns 0.
+ * application continues to provide more data to `rav1e_twopass_in()` in a loop
+ * until `rav1e_twopass_in()` returns 0.
  */
 size_t rav1e_twopass_bytes_needed(RaContext *ctx);
 
@@ -801,8 +825,8 @@ size_t rav1e_twopass_bytes_needed(RaContext *ctx);
  * second pass. On success this returns the number of bytes of that data
  * which were consumed. When encoding the second pass of a two-pass encode,
  * this should be called repeatedly in a loop before every call to
- * rav1e_receive_packet() (including the very first one) until no bytes are
- * consumed, or until twopass_bytes_needed() returns 0. Returns -1 on failure.
+ * `rav1e_receive_packet()` (including the very first one) until no bytes are
+ * consumed, or until `twopass_bytes_needed()` returns 0. Returns -1 on failure.
  */
 int rav1e_twopass_in(RaContext *ctx, uint8_t *buf, size_t buf_size);
 
@@ -810,11 +834,11 @@ int rav1e_twopass_in(RaContext *ctx, uint8_t *buf, size_t buf_size);
  * Send the frame for encoding
  *
  * The function increases the frame internal reference count and it can be passed multiple
- * times to different rav1e_send_frame() with a caveat:
+ * times to different `rav1e_send_frame()` with a caveat:
  *
- * The opaque data, if present, will be moved from the Frame to the context
- * and returned by rav1e_receive_packet in the Packet opaque field or
- * the destructor will be called on rav1e_context_unref if the frame is
+ * The opaque data, if present, will be moved from the `Frame` to the `Context`
+ * and returned by `rav1e_receive_packet` in the `Packet` `opaque` field or
+ * the destructor will be called on `rav1e_context_unref` if the frame is
  * still pending in the encoder.
  *
  * Returns:
@@ -830,7 +854,7 @@ RaEncoderStatus rav1e_send_frame(RaContext *ctx, RaFrame *frame);
 RaEncoderStatus rav1e_last_status(const RaContext *ctx);
 
 /**
- * Return a static string matching the EncoderStatus variant.
+ * Return a static string matching the `EncoderStatus` variant.
  *
  */
 const char *rav1e_status_to_str(RaEncoderStatus status);
@@ -846,7 +870,7 @@ const char *rav1e_status_to_str(RaEncoderStatus status);
 RaEncoderStatus rav1e_receive_packet(RaContext *ctx, RaPacket **pkt);
 
 /**
- * Free the RaPacket.
+ * Free the `RaPacket`.
  */
 void rav1e_packet_unref(RaPacket *pkt);
 
@@ -855,7 +879,7 @@ void rav1e_packet_unref(RaPacket *pkt);
  *
  * Its format is compatible with the AV1 Matroska and ISOBMFF specification.
  *
- * Use rav1e_data_unref() to free it.
+ * Use `rav1e_data_unref()` to free it.
  */
 RaData *rav1e_container_sequence_header(const RaContext *ctx);
 
@@ -867,12 +891,12 @@ RaData *rav1e_container_sequence_header(const RaContext *ctx);
  *
  * The data is copied and this function has to be called for each plane.
  *
- * frame: A frame provided by rav1e_frame_new()
- * plane: The index of the plane starting from 0
- * data: The data to be copied
- * data_len: Length of the buffer
- * stride: Plane line in bytes, including padding
- * bytewidth: Number of bytes per component, either 1 or 2
+ * `frame`: A frame provided by `rav1e_frame_new()`
+ * `plane`: The index of the plane starting from 0
+ * `data`: The data to be copied
+ * `data_len`: Length of the buffer
+ * `stride`: Plane line in bytes, including padding
+ * `bytewidth`: Number of bytes per component, either 1 or 2
  */
 void rav1e_frame_fill_plane(RaFrame *frame,
                             int plane,
@@ -881,6 +905,31 @@ void rav1e_frame_fill_plane(RaFrame *frame,
                             ptrdiff_t stride,
                             int bytewidth);
 
+/**
+ * Extract a frame plane
+ *
+ * This is the reverse of `rav1e_frame_fill_plane()`, primarily used for
+ * extracting the source and reconstruction data from a `RaPacket`.
+ *
+ * Currently the frame contains 3 planes, the first is luminance followed by
+ * chrominance.
+ *
+ * The data is copied out of the frame for a single plane.
+ *
+ * `frame`: A frame provided inside a packet returned by `rav1e_receive_packet()`
+ * `plane`: The index of the plane starting from 0
+ * `data`: The destination for the data
+ * `data_len`: Length of the buffer
+ * `stride`: Plane line in bytes, including padding
+ * `bytewidth`: Number of bytes per component, either 1 or 2
+ */
+void rav1e_frame_extract_plane(const RaFrame *frame,
+                               int plane,
+                               uint8_t *data,
+                               size_t data_len,
+                               ptrdiff_t stride,
+                               int bytewidth);
+
 #ifdef __cplusplus
 } // extern "C"
 #endif // __cplusplus
